VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 1  'Persistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "Node"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
Option Explicit
'Simple Tree and List Node object.
'
'Create a new Node as the root object via "As New Node" and
'create child nodes by calling Create().  Append to the
'children Collection via Add(), remove children via
'Remove().
'
'Name is normally fixed at Create() but can later be
'altered via Rename().  Root nodes ("As New Node") have an
'empty ("") Name.  Note that anonymous nodes cannot be
'renamed if they have a Parent.
'
'Child nodes can be moved between trees by calling Remove()
'on the original tree and then Add() on the new tree:
'
'   Root1.Add Root2.Item("SomeNode").Remove("SubNode")
'
'A subtree can also be pruned out as its own tree via
'Remove():
'
'   Set Subtree = BigTree.Remove("SomeSubTree")
'
'Warning:
'
'   Setting any Node.Value to a non-persistable object
'   will cause errors when a Node (or Tree or List of
'   Nodes) is persisted.
'

Private mChildren As Collection
Private mName As String
Private mParent As Node
Private mValue As Variant

Friend Property Get Children() As Collection
    Set Children = mChildren
End Property

Public Property Get Count() As Long
    Count = mChildren.Count
End Property

'Default property.
Public Property Get Item(ByVal Index As Variant) As Node
Attribute Item.VB_UserMemId = 0
    Set Item = mChildren.Item(Index)
End Property

Public Property Get Name() As String
    Name = mName
End Property

Friend Property Let Name(ByVal RHS As String)
    mName = RHS
    PropertyChanged "Name"
End Property

Public Property Get Parent() As Node
    Set Parent = mParent
End Property

Friend Property Set Parent(ByVal RHS As Node)
    Set mParent = RHS
    PropertyChanged "Parent"
End Property

Public Property Get Value() As Variant
    Value = mValue
End Property

Public Property Let Value(ByVal RHS As Variant)
    mValue = RHS
    PropertyChanged "Value"
End Property

Public Property Set Value(ByVal RHS As Object)
    Set Value = RHS
    PropertyChanged "Value"
End Property

Public Sub Add(ByVal Child As Node)
    If Len(Child.Name) < 1 Then
        mChildren.Add Child
    Else
        On Error Resume Next
        mChildren.Add Child, Child.Name
        If Err Then
            On Error GoTo 0
            Err.Raise &H80049F84, TypeName(Me), "Named child nodes must have unique Names"
        End If
        On Error GoTo 0
    End If
    Set Child.Parent = Me
    PropertyChanged "Children"
End Sub

Public Function Create(ByVal Name As String, Optional ByVal Value As Variant) As Node
    Set Create = New Node
    With Create
        .Name = Name
        If Not (IsMissing(Value) Or IsEmpty(Value)) Then .Value = Value
    End With
End Function

'Hidden, Procedure ID = -4.
Public Function NewEnum() As IUnknown
Attribute NewEnum.VB_UserMemId = -4
Attribute NewEnum.VB_MemberFlags = "40"
   Set NewEnum = mChildren.[_NewEnum]
End Function

Public Function Remove(ByVal Item As Variant) As Node
    Set Remove = mChildren.Item(Item)
    mChildren.Remove Item
    Set Remove.Parent = Nothing
    PropertyChanged "Children"
End Function

Public Sub Rename(ByVal NewName As String)
    If Not (Len(mName) > 0 Or mParent Is Nothing) Then
        Err.Raise &H80049F80, TypeName(Me), "Anonymous child nodes cannot be renamed"
    End If
    
    If Not mParent Is Nothing Then
        On Error Resume Next
        mParent.Children.Add Me, NewName
        If Err Then
            On Error GoTo 0
            Err.Raise &H80049F84, TypeName(Me), "Named child nodes must have unique Names"
        End If
        On Error GoTo 0
        
        mParent.Children.Remove mName
    End If
    mName = NewName
    PropertyChanged "Name"
End Sub

Private Sub Class_Initialize()
    Set mChildren = New Collection
End Sub

Private Sub Class_ReadProperties(PropBag As PropertyBag)
    Dim ChildrenCount As Long
    Dim I As Long
    Dim Child As Node
    
    With PropBag
        'De-persist the Children (Collection is not persistable).
        ChildrenCount = .ReadProperty("ChildrenCount", 0)
        For I = 1 To ChildrenCount
            Add Create(.ReadProperty("ChildName" & CStr(I), ""), _
                       .ReadProperty("ChildValue" & CStr(I), Empty))
        Next
        
        If mParent Is Nothing Then
            'We're a root or free node.
            mName = .ReadProperty("Name", "")
            mValue = .ReadProperty("Value", Empty)
        End If
    End With
End Sub

Private Sub Class_Terminate()
    Set mParent = Nothing   'Remove circular reference.
    Set mChildren = Nothing 'Clean up the Children.
End Sub

Private Sub Class_WriteProperties(PropBag As PropertyBag)
    Dim I As Long
    Dim Child As Node
    
    With PropBag
        'Persist the Children (Collection is not persistable).
        .WriteProperty "ChildrenCount", mChildren.Count, 0
        For I = 1 To mChildren.Count
            Set Child = mChildren.Item(I)
            .WriteProperty "ChildName" & CStr(I), Child.Name, ""
            .WriteProperty "ChildValue" & CStr(I), Child.Value, Empty
        Next
        
        If mParent Is Nothing Then
            'We're a root or free node.
            .WriteProperty "Name", mName, ""
            .WriteProperty "Value", mValue, Empty
        End If
    End With
End Sub
